package svc

import (
	commonUtil "cicv-data-closedloop/common/util"
	commonConfig "cicv-data-closedloop/kinglong/common/cfg"
	"cicv-data-closedloop/kinglong/common/ent"
	"cicv-data-closedloop/kinglong/common/global"
	"cicv-data-closedloop/kinglong/common/log"
	commonService "cicv-data-closedloop/kinglong/common/svc"
	"cicv-data-closedloop/kinglong/common/util"
	"net"
	"time"
)

// RunTimeWindowProducerQueue 将时间窗口内的包全部move出去，并等待当前时间窗口结束触发上传
func RunTimeWindowProducerQueue() {
	log.GlobalLogger.Info("生产者队列goroutine - 启动")
	for {
		// 收到自杀信号
		// TODO 此处也应该清理TCP
		select {
		case signal := <-commonService.ChannelKillMove:
			if signal == 1 {
				commonService.ChannelKillMove <- 1
				if len(global.TimeWindowProducerQueue) == 0 {
					commonService.AddKillTimes("4")
					return
				}
			} else { //signal == 2
				commonService.AddKillTimes("4")
				return
			}
		default:
		}

		time.Sleep(time.Duration(1) * time.Second)
		if len(global.TimeWindowProducerQueue) > 0 {
			bags := util.ListAbsolutePathWithSuffixAndSort(commonConfig.CloudConfig.BagDataDir, ".bag")
			currentTimeWindow := global.TimeWindowProducerQueue[0]
			move := false
			bigger := false
			for _, bag := range bags {
				bagTime := util.GetBagTime(bag)
				// 2 如果bag不小于timeWindowBegin不大于timeWindowEnd，则移动
				compare1 := util.TimeCustom1GreaterEqualThanTimeCustom2(bagTime, currentTimeWindow.TimeWindowBegin)
				compare2 := util.TimeCustom1LessEqualThanTimeCustom2(bagTime, currentTimeWindow.TimeWindowEnd)
				if compare1 && compare2 {
					// 将bag包移动到Copy目录
					util.MoveFromDataToCopy(currentTimeWindow.FaultTime, bag)
					move = true
				} else {
					if util.TimeCustom1GreaterEqualThanTimeCustom2(bagTime, currentTimeWindow.TimeWindowBegin) {
						// 必须已经生成了窗口之后的包才算窗口结束了
						bigger = true
						break
					}
				}
			}
			// 如果没有包可以供当前窗口移动，且已经生成了更新的包，则当前窗口已经可以上传
			if !move && bigger {
				// 1 如果第一个已经大于了timeWindowEnd，则触发上传并删除
				// 将时间窗口发送给从节点
				currentTimeWindow.CanUpload = "yes"
				log.GlobalLogger.Info("将已完成的窗口发送给从节点：", currentTimeWindow.CanUpload)
				util.SupplyCopyBags(currentTimeWindow)
				util.RefreshTcpSendTime()
				go sendTimeWindowByTcp(currentTimeWindow)
				// 将时间窗口移出准备队列
				util.RemoveHeadOfdTimeWindowProducerQueue()
				// 将时间窗口加入运行队列
				util.AddTimeWindowToTimeWindowConsumerQueue(currentTimeWindow)
				// 获取copy目录下的字典json，key为触发时间，value为label
				timeToLabelJson, _ := commonUtil.ReadFile(commonConfig.CloudConfig.TimeToLabelJsonPath)
				timeToLabelMap, _ := commonUtil.JsonStringToMap(timeToLabelJson)
				timeToLabelMap[currentTimeWindow.FaultTime] = commonUtil.ToString(currentTimeWindow.Labels)
				timeToLabelJson, _ = commonUtil.MapToJsonString(timeToLabelMap)
				_ = commonUtil.WriteFile(timeToLabelJson, commonConfig.CloudConfig.TimeToLabelJsonPath)
				continue
			} else { // 保证当前窗口只发送一次，每间隔5秒发一次
				if int(time.Since(global.TcpSendTime).Seconds()) > commonConfig.CloudConfig.TimeWindowSendGap {
					log.GlobalLogger.Info("每隔", commonConfig.CloudConfig.TimeWindowSendGap, "秒发送一次tcp消息")
					util.RefreshTcpSendTime()
					// 2 如果第一个不大于timeWindowEnd，则发送不可上传的窗口信息。
					currentTimeWindow.CanUpload = "no"
					go sendTimeWindowByTcp(currentTimeWindow)
				}
			}
		}
	}
}

//TODO 服务端接受连接时如何维护该链接
//func sendTimeWindowByTcp(timeWindow ent.TimeWindow) {
//	// 发送数据
//	send := util2.TimeWindowToJson(timeWindow)
//	_, err := masterCfg.TcpConnection.Write([]byte(send))
//	if err != nil {
//		log.GlobalLogger.Error("master发送给slave时间窗口", timeWindow, "失败：", err)
//		return
//	}
//}

func sendTimeWindowByTcp(timeWindow ent.TimeWindow) {
	socket := commonConfig.CloudConfig.Hosts[1].Ip + ":" + commonConfig.CloudConfig.TcpPort
	tcpConn, err := net.Dial("tcp", socket)
	if err != nil {
		log.GlobalLogger.Error("建立tcp连接", socket, "失败：", err)
		return
	}
	defer tcpConn.Close()
	// 发送数据
	send := util.TimeWindowToJson(timeWindow)
	_, err = tcpConn.Write([]byte(send))
	if err != nil {
		log.GlobalLogger.Error("master发送给slave时间窗口", timeWindow, "失败：", err)
		return
	}
}
